#!/usr/bin/perl

#
# Exercises SP -> WAYF -> IdP -> SP end-to-end test
# This assumes your Top level web page at the SP has a shib-protected "/secure" link.
# Output is formatted as a single-line in Nagio's acceptable format.
#
# It is likely many IdPs from might work.
# Just change the "idp" argument to be your EntityID
#
# Search below for "PASSWORD" and "USERNAME" to change the defaults
# used by this code.  Similarly search for every occurence of my.org
# and change it accordingly for your situation.  One example is
# urn:mace:incommon:my.org should be changed to your own entityID
#
# Example command line invocations (second example runs the test using 
#                                   default options embedded in the code)
# ./check_sp-test.my.org_shib_login.pl -help
# ./check_sp-test.my.org_shib_login.pl 
# ./check_sp-test.my.org_shib_login.pl -sp https://sp-test.my.org -idp urn:mace:incommon:my.org -username USERNAME -password PASSWORD
#


use strict;
use warnings;
use WWW::Mechanize;
use Getopt::Long qw(:config no_ignore_case);


# The WWW::Mechanize module simulates a browser.  For documentation see
# http://search.cpan.org/~petdance/WWW-Mechanize-1.60/lib/WWW/Mechanize.pm
use WWW::Mechanize;



###
### Utilities
###


# Quits program with appropriate error message understandable by Nagios
sub quit
{
  my ($status, $msg) = @_;
  
  my %stat = ('ok' => 0, 'warning' => 1, 'critical' => 2, 'unknown' => 3);
  $status = 'unknown' unless (exists $stat{$status});
  
  print uc($status) . ': ' . $msg . "\n";
  exit $stat{$status};
} # end quit



# Will be invoked in case of timeout
$SIG{'ALRM'} = sub {
  quit('unknown', 'Timed out');
};


# Exception handler for when a critical error is 
# encountered that wasn't explicitly planned for
sub DIE_handler {
    my($signal) = @_;
    quit('critical', $signal);
}


# If we receive a __DIE__ signal, call our local DIE_handler routine (see above)
$SIG{__DIE__}  = 'DIE_handler';



###
### Options
###

my ($sp, $idp, $username, $password, $timeout, $ptimeout, $help );
my $opt = GetOptions( 'S|sp=s' => \$sp,
                      'I|idp=s' => \$idp,
                      'U|username=s' => \$username,
                      'P|password=s' => \$password,
                      't|timeout=i' => \$timeout,
                      'p|pagetimeout=i' => \$ptimeout,
                      'h|help' => \$help,
                    );

$sp = 'http://sp-test.my.org' unless $sp;
$idp = 'urn:mace:incommon:my.org' unless $idp;
$username = 'USERNAME' unless $username;
$password = 'PASSWORD' unless $password;
$timeout = 10 unless $timeout;
$ptimeout = 3 unless $ptimeout;


if ($help || !$opt || scalar @ARGV)
{
  print <<END;
Usage:
  $0 [options]
  
  Options:
    -S, -sp             Top level SP page (should have a shib "/secure" link)
                        Default: "http://sp-test.my.org"
    
    -I, -idp            Entity ID of the IdP to log in at
                        Default: "urn:mace:incommon:my.org"
    
    -U, -username       Username to log in as
    
    -P, -password       Password to log in with
    
    -t, -timeout        Timeout for script overall, in seconds.
                        Default: 10
    
    -p, -pagetimeout    Timeout for individual requests, in seconds.
                        Default: 3
    
    -h, -help           This usage message.
END
  exit 0 if $help;
  quit('unknown', 'Unrecognized command line arguments');
}




###
### Process
###


# If $timeout seconds goes by and we're not done, send an alarm signal
alarm($timeout);



# Create a new instance of WWW::Mechanize
# autocheck checks each request to ensure it was successful, producing an error if not.
my $mech = WWW::Mechanize->new(after => 1,
                               'timeout' => $timeout,
                               'agent' => 'Shib SP Checker (WWW-Mechanize)');


#############################
# Start at the SP ...
#############################
$mech->get($sp);                               # Retrieve the top level page of sp
my $page = $mech->content;                     # Assign the page content to $page
# print $page;                                 # Output the page
my $link = $mech->find_link( text => '/secure' );
quit('critical', 'Could not find <'.${sp}.'> /secure link!') if (! $link);
my $u1 = $mech->uri;                           # Save URI of current (SP) page
my $h1 = $u1->host;                            # Save host from current (SP) page
$mech->follow_link( text => '/secure' );       # follow the SP's '/secure' link over to the WAYF....




#############################
# ... Now we're at the WAYF 
#############################
my $u2 = $mech->uri;                           # Save URI of current (WAYF) page 
my $h2 = $u2->host;                            # Save host from current (WAYF) page
quit('critical', 'SP and WAYF have same URI="'.$u1.'" but should be different') if ($u1 eq $u2); 
# $page = $mech->content;                      # Assign the page content to $page
#print $page;                                  # Output the page
my $form = $mech->form_number(1);              # Interact with the first (only in this case) form on this page
$mech->set_fields('origin' => $idp);           # Pick our IdP
$mech->click_button('number' => 1);            # Click on the first (only) button on this page (select), sends to IdP....



#############################
# ... Now we're at the IdP 
#############################
my $u3 = $mech->uri;                           # Save URI of current (IdP) page 
my $h3 = $u3->host;                            # Save host from current (IdP) page
quit('critical', 'SP and IdP test stages have same host="'.$h1.'" but should be different (perhaps supplied IdP entityID is not valid?)') if ($h1 eq $h3); 
# $page = $mech->content;                      # Assign the page content to $page
#$page = $mech->content;                       # Assign the page content to $page
#print $page;                                  # Output the page
$mech->submit_form (
        form_number => 1,
        fields => {
            j_username => $username, 
            j_password => $password,
            },
        );                                     # Submit the 1st form after filling in username / password

# Actually if I stop here, I get the following back from the page:
#       :
#   <strong>Note:</strong> Since your browser does not support JavaScript,
#   you must press the Continue button once to proceed.
#       :
# Bizarre!  Adding this submit seems to actually complete the job

#print 'Before username/password submit@IDP, uri='.$mech->uri."\n";           # Show the current uri
$mech->submit();                               # Submits the form and should send us back to the SP....



##############################
# ... Now we're back at the SP
##############################
my $u4 = $mech->uri;                           # Save URI of current (IdP) page 
my $h4 = $u4->host;                            # Save host from current (IdP) page
#print 'After username/password submit@IDP, uri='.$mech->uri."\n";     # Show the current uri
quit('critical', 'It appears the login on the "'.$h3.'" IdP failed') if ($h3 eq $h4); 
quit('critical', 'Final test test stage is at host="'.$h4.'" but should be at host "'.$h1.'"') if ($h1 ne $h4); 
#$page = $mech->content;                       # Assign the page content to $page
#print $page;                                  # Output the page

# Finally, lets check our resulting output page for no Login link (which means success)
$link = $mech->find_link( text => 'Login' );
quit('critical', '<'.${sp}.'> Login link has not gone away after login!') if ($link);
quit('ok', 'SP -> WAYF -> IDP -> SP check succeeded (Using SP="'.$sp.'" and IdP="'.$idp.'")');
